@@ -13,8 +13,7 @@
 #include "ws_symbol_export.h"
 
 #include <epan/conversation.h>
-#include <epan/wmem/wmem.h>
-#include <epan/wmem/wmem_interval_tree.h>
+#include <epan/wmem_scopes.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -49,8 +48,12 @@ struct mptcpheader {
 	gboolean mh_mpc;         /* true if seen an mp_capable option */
 	gboolean mh_join;        /* true if seen an mp_join option */
 	gboolean mh_dss;         /* true if seen a dss */
-	gboolean mh_fastclose;   /* true if seen a fastclose */
+	gboolean mh_add;         /* true if seen an MP_ADD */
+	gboolean mh_remove;      /* true if seen an MP_REMOVE */
+	gboolean mh_prio;        /* true if seen an MP_PRIO */
 	gboolean mh_fail;        /* true if seen an MP_FAIL */
+	gboolean mh_fastclose;   /* true if seen a fastclose */
+	gboolean mh_tcprst;      /* true if seen a MP_TCPRST */
 
 	guint8  mh_capable_flags; /* to get hmac version for instance */
 	guint8  mh_dss_flags; /* data sequence signal flag */
@@ -309,6 +312,14 @@ typedef struct tcp_analyze_seq_flow_info_t {
 				 * fast retransmissions and outoforder
 				 */
 
+	/*
+	 * Handling of SACK blocks
+	 * Copied from tcpheader
+	 */
+	guint8  num_sack_ranges;
+	guint32 sack_left_edge[MAX_TCP_SACK_RANGES];
+	guint32 sack_right_edge[MAX_TCP_SACK_RANGES];
+
 } tcp_analyze_seq_flow_info_t;
 
 	/* Process info, currently discovered via IPFIX */
@@ -323,7 +334,7 @@ typedef struct tcp_process_info_t {
 typedef struct _tcp_flow_t {
 	guint8 static_flags; /* true if base seq set */
 	guint32 base_seq;	/* base seq number (used by relative sequence numbers)*/
-#define TCP_MAX_UNACKED_SEGMENTS 1000 /* The most unacked segments we'll store */
+#define TCP_MAX_UNACKED_SEGMENTS 10000 /* The most unacked segments we'll store */
 	guint32 fin;		/* frame number of the final FIN */
 	guint32 window;		/* last seen window */
 	gint16	win_scale;	/* -1 is we don't know, -2 is window scaling is not used */
@@ -332,6 +343,9 @@ typedef struct _tcp_flow_t {
 	gboolean valid_bif;     /* if lost pkts, disable BiF until ACK is recvd */
 	guint32 push_bytes_sent; /* bytes since the last PSH flag */
 	gboolean push_set_last; /* tracking last time PSH flag was set */
+	guint8 mp_operations; /* tracking of the MPTCP operations */
+	gboolean is_first_ack;  /* indicates if this is the first ACK */
+	gboolean closing_initiator; /* tracking who is responsible of the connection end */
 
 	tcp_analyze_seq_flow_info_t* tcp_analyze_seq_info;
 
@@ -378,6 +392,9 @@ struct mptcp_analysis {
 
 	/* identifier of the tcp stream that saw the initial 3WHS with MP_CAPABLE option */
 	struct tcp_analysis *master;
+
+	/* Keep track of the last TCP operations seen in order to avoid false DUP ACKs */
+	guint8 mp_operations;
 };
 
 struct tcp_analysis {
@@ -456,6 +473,13 @@ struct tcp_analysis {
 	 * can exist without any meta
 	 */
 	struct mptcp_analysis* mptcp_analysis;
+
+	/* Track the TCP conversation completeness, as the capture might
+	 * contain all parts of a TCP flow (establishment, data, clearing) or
+	 * just some parts if we jumped on the bandwagon of an already established
+	 * connection or left before it was terminated explicitly
+	 */
+	guint8          conversation_completeness;
 };
 
 /* Structure that keeps per packet data. First used to be able
@@ -515,7 +539,7 @@ WS_DLL_PUBLIC guint32 get_tcp_stream_count(void);
 WS_DLL_PUBLIC guint32 get_mptcp_stream_count(void);
 
 /* Follow Stream functionality shared with HTTP (and SSL?) */
-extern gchar *tcp_follow_conv_filter(packet_info *pinfo, guint *stream, guint *sub_stream);
+extern gchar *tcp_follow_conv_filter(epan_dissect_t *edt, packet_info *pinfo, guint *stream, guint *sub_stream);
 extern gchar *tcp_follow_index_filter(guint stream, guint sub_stream);
 extern gchar *tcp_follow_address_filter(address *src_addr, address *dst_addr, int src_port, int dst_port);
 
